/*--------------------------------------------
   CRM32Pro SDK - Roberto Prieto
   Copyright (C) 2001-2012 MegaStorm Systems
  --------------------------------------------
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
    Roberto Prieto
    contact@megastormsystems.com
    http://www.megastormsystems.com

    Used software licenses, including the one for this library, are located
    at /licenses directory. Please, include them on your distributions.

 ------------------------------------------------------------------------ */

#ifndef CRM32PRO_DT_H
#define CRM32PRO_DT_H

// ---------------------------------------------------------------------------
/** @weakgroup MODULE16 MODULE 16: DataTypes v1.00
<br>It contains some useful data types and methods to be used on multimedia applications: <br>
<ul> <li>2D vector structure and methods. It is based on the work done by Mat Buckland</li> </ul>
- Created on 7 February 2012
- Last modified on 18 February 2012*/
// ---------------------------------------------------------------------------
//! @{

#include <math.h>

//! CRM32Pro DataTypes
/*! Useful data types and methods.*/
class CRM32Pro_DataTypes
{   
public:

    //! Vector 2D class
    class sV2D
    {
    public:
        double x, y;

        // Constructor
        sV2D(double a = 0.0f, double b = 0.0f)
        {
            x = a; y = b;
        }

        // Overloaded operators
        sV2D &operator+=(const sV2D &rhs)
        {
            x += rhs.x;
            y += rhs.y;
            return *this;
        }
        sV2D &operator-=(const sV2D &rhs)
        {
            x -= rhs.x;
            y -= rhs.y;
            return *this;
        }
        sV2D &operator*=(const double &rhs)
        {
            x *= rhs;
            y *= rhs;
            return *this;
        }
        sV2D &operator/=(const double &rhs)
        {
            x /= rhs;
            y /= rhs;
            return *this;
        }
        bool operator==(const sV2D& rhs)
        {
            return ((x == rhs.x) && (y == rhs.y));
        }
        bool operator!=(const sV2D& rhs)
        {
            return (x != rhs.x) || (y != rhs.y);
        }

        // Methods
        //! Length of the 2D vector
        /*! \return double with the length of the vector */
        inline double Length()
        {
            return sqrt(x * x + y * y);
        }

        //! Normalizes the 2D Vector
        inline void Normalize()
        {
            double vector_length = Length();
            x = x / vector_length;
            y = y / vector_length;
        }

        //! Calculates the dot product
        /*! \param  v2 Vector
        \return double with the Dot product */
        inline double Dot(sV2D &v2)
        {
            return x*v2.x + y*v2.y;
        }

        //! Calculates the sign
        /*! \param  v2 Vector
        \return positive integer if v2 is clockwise of us, minus if anticlockwise */
        inline int Sign(sV2D &v2)
        {
            if (y * v2.x > x * v2.y) return 1;
            else return -1;
        }

        //! Get a perpendicular vector
        /*! \return sV2D perpendicular to this one */
        inline sV2D GetPerpendicular()
        {
            return sV2D(-y, x);
        }

        //! Adjusts x and y so that the length of the vector does not exceed max
        /*! \param max double with the maximum length */
        inline void Truncate(double max)
        {
            double dL = Length();

            if (dL > max)
            {
                // Normalize
                x = x / dL;
                y = y / dL;
                // Multiply for our max
                x *= max; y *= max;
            } 
        }

        //! Distance between this vector and the one passed as a parameter
        /*! \param  v2 Vector
        \return double with the distance between both vectors */
        inline double Distance(const sV2D &v2)
        {
            double ySeparation = v2.y - y;
            double xSeparation = v2.x - x;
            return sqrt(ySeparation*ySeparation + xSeparation*xSeparation);
        }

        // Get a reflected vector.
        /*! \param norm normalized vector (the wall?)
        \return sV2D with the reflected vector */
        inline sV2D GetReflect(sV2D& norm)
        {
            sV2D sAux = norm;
            sAux *= Dot(norm) * 2.0;			
            return sV2D(x - sAux.x, y - sAux.y);
        }

        //! Get the reverse vector
        inline sV2D GetReverse()
        {
            return sV2D(-x, -y);
        }

    };
};

#define CDT CRM32Pro_DataTypes

// Overloaded operators for Vector 2D
inline CDT::sV2D operator*(const CDT::sV2D &lhs, double rhs)
{
    CDT::sV2D result(lhs);
    result *= rhs;
    return result;
}
inline CDT::sV2D operator*(double lhs, const CDT::sV2D &rhs)
{
    CDT::sV2D result(rhs);
    result *= lhs;
    return result;
}
inline CDT::sV2D operator/(const CDT::sV2D &lhs, double rhs)
{
    CDT::sV2D result(lhs);
    result /= rhs;
    return result;
}
inline CDT::sV2D operator-(const CDT::sV2D &lhs, const CDT::sV2D &rhs)
{
    CDT::sV2D result(lhs);
    result.x -= rhs.x;
    result.y -= rhs.y;  
    return result;
}
inline CDT::sV2D operator+(const CDT::sV2D &lhs, const CDT::sV2D &rhs)
{
    CDT::sV2D result(lhs);
    result.x += rhs.x;
    result.y += rhs.y;  
    return result;
}
//! @}
#endif